#include <net/socket.h>
#include <net/ethernet.h>
#include <net/net_core.h>
#include <net/net_l2.h>
#include <net/net_if.h>
#include <net/gptp.h>
#include <posix/pthread.h>
#include <sys/util.h>
#include <sys/printk.h>
#include <posix/unistd.h>
#include <drivers/gpio.h>
#include <drivers/counter.h>
#include <kernel.h>

#include <zephyr.h>
#include <device.h>
#include <errno.h>
#include <stdbool.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "avtp.h"
#include "avtp_stream.h"
#include "gyro.h"

//#define RTC_DEV			DT_LABEL(DT_NODELABEL(ftm1))
//#define ALARM_CHANNEL_ID	0//DT_IRQ_BY_IDX(DT_NODELABEL(rtc),43,irq)
#define STREAM_ID		0x02049f36bf541234
#define MOCK_TIMESTAMP		0x12345678
#define SAMPLE_SIZE		2 /*in bytes*/
#define NUM_AXIS		3
#define DATA_LEN		(SAMPLE_SIZE * NUM_AXIS)
#define PDU_SIZE		(sizeof(struct avtp_stream_pdu) + DATA_LEN)
#define I2C_DEV 		DT_LABEL(DT_NODELABEL(i2c0))
#define PTC12_LABEL		DT_GPIO_LABEL(DT_NODELABEL(ptc12_but_cool), gpios)
#define PTC12_PIN		DT_GPIO_PIN(DT_NODELABEL(ptc12_but_cool), gpios)
#define GYRO_INT_FLAGS		(GPIO_INPUT | DT_GPIO_FLAGS(DT_NODELABEL(ptc12), gpios))

K_SEM_DEFINE(gyro_data_rdy,0,1);
K_SEM_DEFINE(cbs_timer_sem,0,1);
K_SEM_DEFINE(transmitAllowed,0,1);
K_MUTEX_DEFINE(g_data_access);

int s; //socket
int16_t g_data[3] = {0};
uint64_t avtptime;

//struct counter_top_cfg rtc_cfg;
//struct counter_alarm_cfg rtc_cfg;
struct k_timer cbs_timer;

void cbs_timeout(struct k_timer *timer_id){
	k_sem_give(&cbs_timer_sem);
}

static struct gpio_callback gyro_cb_data;
void gyro_interrupt(const struct device *dev, struct gpio_callback *cb, uint32_t pins)
{
	k_sem_give(&gyro_data_rdy);
}


static int init_pdu(struct avtp_stream_pdu *pdu)
{
	int res;

	res = avtp_stream_pdu_init(pdu);
	if (res < 0)
		return -1;

	res = avtp_stream_pdu_set(pdu, AVTP_STREAM_FIELD_TV, 0);
	if (res < 0)
		return -1;

	res = avtp_stream_pdu_set(pdu, AVTP_STREAM_FIELD_STREAM_ID, STREAM_ID);
	if (res < 0)
		return -1;

	res = avtp_stream_pdu_set(pdu, AVTP_STREAM_FIELD_STREAM_DATA_LEN, DATA_LEN);
	if (res < 0)
		return -1;

	res = avtp_stream_pdu_set(pdu, AVTP_STREAM_FIELD_TIMESTAMP, MOCK_TIMESTAMP);
	if (res < 0)
		return -1;

	res = avtp_stream_pdu_set(pdu, AVTP_STREAM_FIELD_SEQ_NUM, 0);
	return 0;
}

void packet_sender_thread(void)
{
	//const struct device *rtc_dev;
	//rtc_dev = device_get_binding(RTC_DEV);
	//counter_start(rtc_dev);

	//rtc_cfg.flags = 0;
	//rtc_cfg.ticks = counter_us_to_ticks(rtc_dev, 2500);
	//rtc_cfg.callback = rtc_interrupt;
	//rtc_cfg.user_data = &rtc_cfg;
	
	//int ret;
	//ret = counter_set_top_value(rtc_dev, &rtc_cfg);
	//printk("top value return: %i\n", ret);
	//counter_start(rtc_dev);
	//printk("timer setup complete\n");

	//counter_set_channel_alarm(rtc_dev, ALARM_CHANNEL_ID, &rtc_cfg);
	//uint32_t freq;
	//freq = counter_get_frequency(rtc_dev);
	//uint8_t chan_num;
	//chan_num = counter_get_num_of_channels(rtc_dev);
	//printk("freq: %i chan_num: %i\n", freq, chan_num);

	k_timer_init(&cbs_timer, cbs_timeout, NULL);
	
	k_sleep(K_SECONDS(1));

	k_timer_start(&cbs_timer, K_USEC(250), K_USEC(100));
	
	const unsigned char ether_broadcast_addr[]={0x00,0x1b,0x21,0xe4,0x67,0x07};
	uint8_t seq_num = 0;
	struct avtp_stream_pdu *pdu = alloca(PDU_SIZE);
	
	if ((s = zsock_socket(AF_PACKET, SOCK_DGRAM, 0x22f0)) < 0) {
		printk("cannot create socket\n%i\n",s);
		return;
	}
	
	struct sockaddr_ll addr;
	addr.sll_family = AF_PACKET;
	addr.sll_ifindex = 1;
	addr.sll_protocol = 0xf022;
	addr.sll_halen = sizeof("xx:xx:xx:xx:xx:xx");
	memcpy(addr.sll_addr,ether_broadcast_addr,sizeof("xx:xx:xx:xx:xx:xx"));
	
	init_pdu(pdu);
	
	k_sleep(K_SECONDS(8));

	avtp_stream_pdu_set(pdu, AVTP_STREAM_FIELD_TV, 1);
	while(1){
		k_sem_take(&transmitAllowed, K_FOREVER);

		k_mutex_lock(&g_data_access,K_FOREVER);
		memcpy(pdu->avtp_payload,g_data,DATA_LEN);
		avtp_stream_pdu_set(pdu, AVTP_STREAM_FIELD_TIMESTAMP, avtptime);
		k_mutex_unlock(&g_data_access);

		zsock_sendto(s, pdu, PDU_SIZE, 0, (struct sockaddr *)&addr, sizeof(addr));

		avtp_stream_pdu_set(pdu, AVTP_STREAM_FIELD_SEQ_NUM, ++seq_num);
	}

}

void credit_based_shaper(void)
{
	int portTransmitRate = 100000000;	//bits per second on port
	int idleSlope = 1920;			//bits per second used by avtp
	int sendSlope = idleSlope - portTransmitRate;
	int credit = 0; //
	bool transmit = false;
	//bool transmitAllowed = true;
	while(1){
		k_sem_take(&cbs_timer_sem, K_FOREVER);
		credit += 480;
		if(credit >= 0) {
			credit -= 595200;//595238;//600000; // fine-tuning shaper
			k_sem_give(&transmitAllowed);
		}
	}
}

void data_collector_thread(void)
{
	bool gm_present;
	struct net_ptp_time ptpts;

	const struct device *i2c_dev;
	i2c_dev = device_get_binding(I2C_DEV);
	if (!i2c_dev) {
		printk("i2c driver not found\n");
	}
	k_sleep(K_SECONDS(1));
	gyro_init(i2c_dev);

	const struct device *int_port;
	int_port = device_get_binding(PTC12_LABEL);
	if (!int_port) {
                printk("could not bind port\n");
        }
	int ret;
	ret = gpio_pin_configure(int_port, PTC12_PIN, GYRO_INT_FLAGS);
	if (ret != 0) {
                printk("Error %d: failed to configure pin\n", ret);
	}
	ret = gpio_pin_interrupt_configure(int_port, PTC12_PIN, GPIO_INT_EDGE_TO_ACTIVE);
	if (ret != 0) {
                printk("Error %d: failed to configure interrupt\n", ret);
        }
	gpio_init_callback(&gyro_cb_data, gyro_interrupt, BIT(PTC12_PIN));
	ret = gpio_add_callback(int_port, &gyro_cb_data);
	if (ret != 0) {
		printk("Error %d: failed to configure callback\n", ret);
	}
	printk("interupt initialized\n");

	k_mutex_lock(&g_data_access,K_FOREVER);
	gyro_read(i2c_dev,g_data);
	k_mutex_unlock(&g_data_access);

	k_sleep(K_SECONDS(1));
	while(1){
		k_sem_take(&gyro_data_rdy, K_FOREVER);

		k_mutex_lock(&g_data_access,K_FOREVER);
		gptp_event_capture(&ptpts, &gm_present);
		gyro_read(i2c_dev,g_data);
		avtptime = ((ptpts.second * NSEC_PER_SEC) + ptpts.nanosecond) % (1ULL << 32);
		k_mutex_unlock(&g_data_access);
	}
}

void xXx_RX_rens_6000_xXx(void)
{
	char endlesspacketdrain[128];
	while(1){
		zsock_recv(s,endlesspacketdrain,128,ZSOCK_MSG_DONTWAIT);
		k_sleep(K_MSEC(25));
	}
}

K_THREAD_DEFINE(PACKET_DELIVERY,1024,packet_sender_thread,NULL,NULL,NULL,5,0,0);
K_THREAD_DEFINE(GYROSCOPE_READER,1024,data_collector_thread,NULL,NULL,NULL,6,0,1000);
K_THREAD_DEFINE(BUFFER_DESTROYER,1024,xXx_RX_rens_6000_xXx,NULL,NULL,NULL,7,0,8000);
K_THREAD_DEFINE(CREDITBASEDSHAPER,1024,credit_based_shaper,NULL,NULL,NULL,5,0,9000);



